@@ -1,5 +1,5 @@
 /*
-* "$Id: usb-darwin.c 7319 2008-02-15 23:26:51Z mike $"
+* "$Id: usb-darwin.c 8207 2009-01-07 17:51:00Z mike $"
 *
 * Copyright 2005-2008 Apple Inc. All rights reserved.
 *
@@ -92,6 +92,7 @@
 #include <cups/debug.h>
 #include <cups/sidechannel.h>
 #include <cups/i18n.h>
+#include "backend-private.h"
 
 #include <CoreFoundation/CoreFoundation.h>
 #include <IOKit/usb/IOUSBLib.h>
@@ -632,14 +633,12 @@ print_device(const char *uri,		/* I - Device URI */
 	status = (*g.classdriver)->WritePipe(g.classdriver, (UInt8*)print_ptr, &bytes, 0);
 
        /*
-	* Ignore timeout errors...
+	* Ignore timeout errors, but retain the number of bytes written to
+	* avoid sending duplicate data (<rdar://problem/6254911>)...
 	*/
 
 	if (status == kIOUSBTransactionTimeout)
-	{
 	  status = 0;
-	  bytes = 0;
-	}
 
 	if (status || bytes < 0)
 	{
@@ -1008,25 +1007,23 @@ static Boolean list_device_cb(void *refcon,
     {
       CFStringRef make = NULL,  model = NULL, serial = NULL;
       char uristr[1024], makestr[1024], modelstr[1024], serialstr[1024];
-      char optionsstr[1024], idstr[1024];
+      char optionsstr[1024], idstr[1024], make_modelstr[1024];
 
       copy_deviceinfo(deviceIDString, &make, &model, &serial);
+      CFStringGetCString(deviceIDString, idstr, sizeof(idstr),
+                         kCFStringEncodingUTF8);
+      backendGetMakeModel(idstr, make_modelstr, sizeof(make_modelstr));
 
       modelstr[0] = '/';
 
-      CFStringGetCString(deviceIDString, idstr, sizeof(idstr),
-                         kCFStringEncodingUTF8);
-
-      if (make)
-        CFStringGetCString(make, makestr, sizeof(makestr),
-	                   kCFStringEncodingUTF8);
-      else
+      if (!make ||
+          !CFStringGetCString(make, makestr, sizeof(makestr),
+			      kCFStringEncodingUTF8))
         strcpy(makestr, "Unknown");
 
-      if (model)
-	CFStringGetCString(model, &modelstr[1], sizeof(modelstr)-1,
-      			   kCFStringEncodingUTF8);
-      else
+      if (!model ||
+          !CFStringGetCString(model, &modelstr[1], sizeof(modelstr)-1,
+			      kCFStringEncodingUTF8))
         strcpy(modelstr + 1, "Printer");
 
       optionsstr[0] = '\0';
@@ -1041,18 +1038,8 @@ static Boolean list_device_cb(void *refcon,
       httpAssembleURI(HTTP_URI_CODING_ALL, uristr, sizeof(uristr), "usb", NULL, makestr, 0, modelstr);
       strncat(uristr, optionsstr, sizeof(uristr));
 
-     /*
-      * Fix common HP 1284 bug...
-      */
-
-      if (!strcasecmp(makestr, "Hewlett-Packard"))
-        strcpy(makestr, "HP");
-
-      if (!strncasecmp(modelstr + 1, "hp ", 3))
-        _cups_strcpy(modelstr + 1, modelstr + 4);
-
-      printf("direct %s \"%s %s\" \"%s %s USB\" \"%s\"\n", uristr, makestr,
-             &modelstr[1], makestr, &modelstr[1], idstr);
+      printf("direct %s \"%s\" \"%s USB\" \"%s\"\n", uristr, make_modelstr,
+             make_modelstr, idstr);
 
       release_deviceinfo(&make, &model, &serial);
       CFRelease(deviceIDString);
@@ -1189,8 +1176,10 @@ static void copy_deviceinfo(CFStringRef deviceIDString,
 
   if (make != NULL)
     *make = copy_value_for_key(deviceIDString, makeKeys);
+
   if (model != NULL)
     *model = copy_value_for_key(deviceIDString, modelKeys);
+
   if (serial != NULL)
     *serial = copy_value_for_key(deviceIDString, serialKeys);
 }
@@ -2039,5 +2028,5 @@ static void get_device_id(cups_sc_status_t *status,
 
 
 /*
- * End of "$Id: usb-darwin.c 7319 2008-02-15 23:26:51Z mike $".
+ * End of "$Id: usb-darwin.c 8207 2009-01-07 17:51:00Z mike $".
  */
