@@ -1,5 +1,5 @@
 /*
- * "$Id: admin.c 7302 2008-02-13 01:15:29Z mike $"
+ * "$Id: admin.c 7984 2008-09-26 15:30:29Z mike $"
  *
  *   Administration CGI for the Common UNIX Printing System (CUPS).
  *
@@ -308,6 +308,16 @@ do_add_rss_subscription(http_t *http)	/* I - HTTP connection */
     return;
   }
 
+ /*
+  * Make sure we have a username...
+  */
+
+  if ((user = getenv("REMOTE_USER")) == NULL)
+  {
+    puts("Status: 401\n");
+    exit(0);
+  }
+
  /*
   * Validate the subscription name...
   */
@@ -352,9 +362,6 @@ do_add_rss_subscription(http_t *http)	/* I - HTTP connection */
     ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri",
                  NULL, printer_uri);
 
-  if ((user = getenv("REMOTE_USER")) == NULL)
-    user = "guest";
-
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "requesting-user-name",
                NULL, user);
 
@@ -897,7 +904,7 @@ do_am_printer(http_t *http,		/* I - HTTP connection */
     cgiCopyTemplateLang("choose-serial.tmpl");
     cgiEndHTML();
   }
-  else if (!file && (var = cgiGetVariable("PPD_NAME")) == NULL)
+  else if (!file && !cgiGetVariable("PPD_NAME"))
   {
     if (modify)
     {
@@ -1268,6 +1275,16 @@ do_cancel_subscription(http_t *http)/* I - HTTP connection */
     return;
   }
 
+ /*
+  * Require a username...
+  */
+
+  if ((user = getenv("REMOTE_USER")) == NULL)
+  {
+    puts("Status: 401\n");
+    exit(0);
+  }
+
  /*
   * Cancel the subscription...
   */
@@ -1279,9 +1296,6 @@ do_cancel_subscription(http_t *http)/* I - HTTP connection */
   ippAddInteger(request, IPP_TAG_OPERATION, IPP_TAG_INTEGER,
                 "notify-subscription-id", id);
 
-  if ((user = getenv("REMOTE_USER")) == NULL)
-    user = "guest";
-
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "requesting-user-name",
                NULL, user);
 
@@ -2584,6 +2598,9 @@ do_set_allowed_users(http_t *http)	/* I - HTTP connection */
       while (*ptr == ',' || isspace(*ptr & 255))
 	ptr ++;
 
+      if (!*ptr)
+        break;
+
       if (*ptr == '\'' || *ptr == '\"')
       {
        /*
@@ -2649,6 +2666,9 @@ do_set_allowed_users(http_t *http)	/* I - HTTP connection */
         while (*ptr == ',' || isspace(*ptr & 255))
 	  ptr ++;
 
+        if (!*ptr)
+	  break;
+
         if (*ptr == '\'' || *ptr == '\"')
 	{
 	 /*
@@ -2998,15 +3018,15 @@ do_set_options(http_t *http,		/* I - HTTP connection */
 
         cgiSetVariable("KEYWORD", "job_sheets_start");
 	cgiSetVariable("KEYTEXT", cgiText(_("Starting Banner")));
-        cgiSetVariable("DEFCHOICE", attr == NULL ?
-	                            "" : attr->values[0].string.text);
+        cgiSetVariable("DEFCHOICE", attr != NULL ?
+	                            attr->values[0].string.text : "");
 
 	cgiCopyTemplateLang("option-pickone.tmpl");
 
         cgiSetVariable("KEYWORD", "job_sheets_end");
 	cgiSetVariable("KEYTEXT", cgiText(_("Ending Banner")));
-        cgiSetVariable("DEFCHOICE", attr == NULL && attr->num_values > 1 ?
-	                            "" : attr->values[1].string.text);
+        cgiSetVariable("DEFCHOICE", attr != NULL && attr->num_values > 1 ?
+	                            attr->values[1].string.text : "");
 
 	cgiCopyTemplateLang("option-pickone.tmpl");
 
@@ -3193,7 +3213,7 @@ do_set_options(http_t *http,		/* I - HTTP connection */
       }
 
       if ((var = cgiGetVariable("protocol")) != NULL)
-	cupsFilePrintf(out, "*cupsProtocol: %s\n", cgiGetVariable("protocol"));
+	cupsFilePrintf(out, "*cupsProtocol: %s\n", var);
 
       cupsFileClose(in);
       cupsFileClose(out);
@@ -3232,12 +3252,12 @@ do_set_options(http_t *http,		/* I - HTTP connection */
     attr->values[1].string.text = _cupsStrAlloc(cgiGetVariable("job_sheets_end"));
 
     if ((var = cgiGetVariable("printer_error_policy")) != NULL)
-      attr = ippAddString(request, IPP_TAG_PRINTER, IPP_TAG_NAME,
-                          "printer-error-policy", NULL, var);
+      ippAddString(request, IPP_TAG_PRINTER, IPP_TAG_NAME,
+		   "printer-error-policy", NULL, var);
 
     if ((var = cgiGetVariable("printer_op_policy")) != NULL)
-      attr = ippAddString(request, IPP_TAG_PRINTER, IPP_TAG_NAME,
-                          "printer-op-policy", NULL, var);
+      ippAddString(request, IPP_TAG_PRINTER, IPP_TAG_NAME,
+		   "printer-op-policy", NULL, var);
 
    /*
     * Do the request and get back a response...
@@ -3431,5 +3451,5 @@ match_string(const char *a,		/* I - First string */
 
     
 /*
- * End of "$Id: admin.c 7302 2008-02-13 01:15:29Z mike $".
+ * End of "$Id: admin.c 7984 2008-09-26 15:30:29Z mike $".
  */
