@@ -1,9 +1,9 @@
 /*
- * "$Id: auth.c 7319 2008-02-15 23:26:51Z mike $"
+ * "$Id: auth.c 8342 2009-02-07 05:06:11Z mike $"
  *
  *   Authorization routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   This file contains Kerberos support code, copyright 2006 by
@@ -111,7 +111,8 @@ static int		compare_locations(cupsd_location_t *a,
 static char		*cups_crypt(const char *pw, const char *salt);
 #endif /* !HAVE_LIBPAM && !HAVE_USERSEC_H */
 #ifdef HAVE_GSSAPI
-static gss_cred_id_t	get_gss_creds(const char *service_name);
+static gss_cred_id_t	get_gss_creds(const char *service_name,
+			              const char *con_server_name);
 #endif /* HAVE_GSSAPI */
 static char		*get_md5_password(const char *username,
 			                  const char *group, char passwd[33]);
@@ -340,7 +341,7 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
   char		*ptr,			/* Pointer into string */
 		username[256],		/* Username string */
 		password[33];		/* Password string */
-  const char	*localuser;		/* Certificate username */
+  cupsd_cert_t	*localuser;		/* Certificate username */
   char		nonce[HTTP_MAX_VALUE],	/* Nonce value from client */
 		md5[33],		/* MD5 password */
 		basicmd5[33];		/* MD5 of Basic password */
@@ -450,25 +451,22 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
       return;
     }
 
-    if ((status = AuthorizationCopyInfo(con->authref, 
-					kAuthorizationEnvironmentUsername, 
-					&authinfo)) != 0)
+    strlcpy(username, "_AUTHREF_", sizeof(username));
+
+    if (!AuthorizationCopyInfo(con->authref, kAuthorizationEnvironmentUsername, 
+			       &authinfo))
     {
-      cupsdLogMessage(CUPSD_LOG_ERROR,
-		      "AuthorizationCopyInfo returned %d (%s)",
-		      (int)status, cssmErrorString(status));
-      return;
+      if (authinfo->count == 1 && authinfo->items[0].value &&
+          authinfo->items[0].valueLength >= 2)
+        strlcpy(username, authinfo->items[0].value, sizeof(username));
+
+      AuthorizationFreeItemSet(authinfo);
     }
-  
-    if (authinfo->count == 1)
-      strlcpy(username, authinfo->items[0].value, sizeof(username));
 
     cupsdLogMessage(CUPSD_LOG_DEBUG,
                     "cupsdAuthorize: Authorized as %s using AuthRef",
 		    username);
 
-    AuthorizationFreeItemSet(authinfo);
-
     con->type = CUPSD_AUTH_BASIC;
   }
 #endif /* HAVE_AUTHORIZATION_H */
@@ -542,7 +540,7 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
 
     if ((localuser = cupsdFindCert(authorization)) != NULL)
     {
-      strlcpy(username, localuser, sizeof(username));
+      strlcpy(username, localuser->username, sizeof(username));
 
       cupsdLogMessage(CUPSD_LOG_DEBUG,
 		      "cupsdAuthorize: Authorized as %s using Local",
@@ -556,7 +554,12 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
       return;
     }
 
-    con->type = CUPSD_AUTH_BASIC;
+#ifdef HAVE_GSSAPI
+    if (localuser->ccache)
+      con->type = CUPSD_AUTH_NEGOTIATE;
+    else
+#endif /* HAVE_GSSAPI */
+      con->type = CUPSD_AUTH_BASIC;
   }
   else if (!strncmp(authorization, "Basic", 5))
   {
@@ -659,7 +662,6 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
 	      cupsdLogMessage(CUPSD_LOG_ERROR,
 	                      "cupsdAuthorize: pam_start() returned %d (%s)!\n",
         	              pamerr, pam_strerror(pamh, pamerr));
-	      pam_end(pamh, 0);
 	      return;
 	    }
 
@@ -975,7 +977,7 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
     * Get the server credentials...
     */
 
-    if ((server_creds = get_gss_creds(GSSServiceName)) == NULL)
+    if ((server_creds = get_gss_creds(GSSServiceName, con->servername)) == NULL)
       return;	
 
    /*
@@ -2435,7 +2437,9 @@ cups_crypt(const char *pw,		/* I - Password string */
  */
 
 static gss_cred_id_t			/* O - Server credentials */
-get_gss_creds(const char *service_name)	/* I - Service name */
+get_gss_creds(
+    const char *service_name, 		/* I - Service name */
+    const char *con_server_name)	/* I - Hostname of server */
 {
   OM_uint32	major_status,		/* Major status code */
 		minor_status;		/* Minor status code */
@@ -2443,12 +2447,10 @@ get_gss_creds(const char *service_name)	/* I - Service name */
   gss_cred_id_t	server_creds;		/* Server credentials */
   gss_buffer_desc token = GSS_C_EMPTY_BUFFER;
 					/* Service name token */
-  char		buf[1024],		/* Service name buffer */
-		fqdn[HTTP_MAX_URI];	/* Hostname of server */
+  char		buf[1024];		/* Service name buffer */
 
 
-  snprintf(buf, sizeof(buf), "%s@%s", service_name,
-	   httpGetHostname(NULL, fqdn, sizeof(fqdn)));
+  snprintf(buf, sizeof(buf), "%s@%s", service_name, con_server_name);
 
   token.value  = buf;
   token.length = strlen(buf);
@@ -2681,5 +2683,5 @@ to64(char          *s,			/* O - Output string */
 
 
 /*
- * End of "$Id: auth.c 7319 2008-02-15 23:26:51Z mike $".
+ * End of "$Id: auth.c 8342 2009-02-07 05:06:11Z mike $".
  */
