@@ -1,9 +1,9 @@
 /*
- * "$Id: main.c 7299 2008-02-12 01:00:19Z mike $"
+ * "$Id: main.c 8301 2009-01-28 22:12:18Z mike $"
  *
  *   Scheduler main loop for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2009 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -133,7 +133,8 @@ main(int  argc,				/* I - Number of command-line args */
 			browse_time,	/* Next browse send time */
 			senddoc_time,	/* Send-Document time */
 			expire_time,	/* Subscription expire time */
-			report_time;	/* Malloc/client/job report time */
+			report_time,	/* Malloc/client/job report time */
+			event_time;	/* Last time an event notification was done */
   long			timeout;	/* Timeout for cupsdDoSelect() */
   struct rlimit		limit;		/* Runtime limit */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
@@ -632,6 +633,23 @@ main(int  argc,				/* I - Number of command-line args */
   cupsdStartSystemMonitor();
 #endif /* __APPLE__ */
 
+ /*
+  * Send server-started event...
+  */
+
+#ifdef HAVE_LAUNCHD
+  if (Launchd)
+    cupsdAddEvent(CUPSD_EVENT_SERVER_STARTED, NULL, NULL,
+                  "Scheduler started via launchd.");
+  else
+#endif /* HAVE_LAUNCHD */
+  if (fg)
+    cupsdAddEvent(CUPSD_EVENT_SERVER_STARTED, NULL, NULL,
+                  "Scheduler started in foreground.");
+  else
+    cupsdAddEvent(CUPSD_EVENT_SERVER_STARTED, NULL, NULL,
+                  "Scheduler started in background.");
+
  /*
   * Start any pending print jobs...
   */
@@ -642,11 +660,13 @@ main(int  argc,				/* I - Number of command-line args */
   * Loop forever...
   */
 
-  browse_time   = time(NULL);
-  expire_time   = time(NULL);
+  current_time  = time(NULL);
+  browse_time   = current_time;
+  event_time    = current_time;
+  expire_time   = current_time;
   fds           = 1;
   report_time   = 0;
-  senddoc_time  = time(NULL);
+  senddoc_time  = current_time;
 
   while (!stop_scheduler)
   {
@@ -738,6 +758,13 @@ main(int  argc,				/* I - Number of command-line args */
         */
 
         cupsdStartServer();
+
+       /*
+        * Send a server-restarted event...
+	*/
+
+        cupsdAddEvent(CUPSD_EVENT_SERVER_RESTARTED, NULL, NULL,
+                      "Scheduler restarted.");
       }
     }
 
@@ -750,7 +777,8 @@ main(int  argc,				/* I - Number of command-line args */
     * times.
     */
 
-    timeout = select_timeout(fds);
+    if ((timeout = select_timeout(fds)) > 1 && LastEvent)
+      timeout = 1;
 
 #if HAVE_LAUNCHD
    /*
@@ -906,7 +934,7 @@ main(int  argc,				/* I - Number of command-line args */
 #endif /* HAVE_LDAP */
     }
 
-    if (Browsing && BrowseLocalProtocols && current_time > browse_time)
+    if (Browsing && current_time > browse_time)
     {
       cupsdSendBrowseList();
       browse_time = current_time;
@@ -1023,7 +1051,7 @@ main(int  argc,				/* I - Number of command-line args */
     * accumulated.  Don't send these more than once a second...
     */
 
-    if (LastEvent)
+    if (LastEvent && (current_time - event_time) >= 1)
     {
 #ifdef HAVE_NOTIFY_POST
       if (LastEvent & (CUPSD_EVENT_PRINTER_ADDED |
@@ -1056,7 +1084,8 @@ main(int  argc,				/* I - Number of command-line args */
       * Reset the accumulated events...
       */
 
-      LastEvent = CUPSD_EVENT_NONE;
+      LastEvent  = CUPSD_EVENT_NONE;
+      event_time = current_time;
     }
   }
 
@@ -1065,10 +1094,18 @@ main(int  argc,				/* I - Number of command-line args */
   */
 
   if (stop_scheduler)
+  {
     cupsdLogMessage(CUPSD_LOG_INFO, "Scheduler shutting down normally.");
+    cupsdAddEvent(CUPSD_EVENT_SERVER_STOPPED, NULL, NULL,
+                  "Scheduler shutting down normally.");
+  }
   else
+  {
     cupsdLogMessage(CUPSD_LOG_ERROR,
                     "Scheduler shutting down due to program error.");
+    cupsdAddEvent(CUPSD_EVENT_SERVER_STOPPED, NULL, NULL,
+                  "Scheduler shutting down due to program error.");
+  }
 
  /*
   * Close all network clients...
@@ -1116,8 +1153,7 @@ main(int  argc,				/* I - Number of command-line args */
     krb5_free_context(KerberosContext);
 #endif /* HAVE_GSSAPI */
 
-#ifdef __APPLE__
-#ifdef HAVE_DLFCN_H
+#if defined(__APPLE__) && defined(HAVE_DLFCN_H)
  /* 
   * Unload Print Service quota enforcement library (X Server only) 
   */
@@ -1128,8 +1164,7 @@ main(int  argc,				/* I - Number of command-line args */
     dlclose(PSQLibRef);
     PSQLibRef = NULL;
   }
-#endif /* HAVE_DLFCN_H */
-#endif	/* __APPLE__ */
+#endif /* __APPLE__ && HAVE_DLFCN_H */
 
 #ifdef __sgi
  /*
@@ -1596,27 +1631,6 @@ process_children(void)
 
     cupsdFinishProcess(pid, name, sizeof(name));
 
-    if (status == SIGTERM)
-      status = 0;
-
-    if (status)
-    {
-      if (WIFEXITED(status))
-	cupsdLogMessage(CUPSD_LOG_ERROR, "PID %d (%s) stopped with status %d!",
-	                pid, name, WEXITSTATUS(status));
-      else
-	cupsdLogMessage(CUPSD_LOG_ERROR, "PID %d (%s) crashed on signal %d!",
-	                pid, name, WTERMSIG(status));
-
-      if (LogLevel < CUPSD_LOG_DEBUG)
-        cupsdLogMessage(CUPSD_LOG_INFO,
-	                "Hint: Try setting the LogLevel to \"debug\" to find "
-			"out more.");
-    }
-    else
-      cupsdLogMessage(CUPSD_LOG_DEBUG, "PID %d (%s) exited with no errors.",
-                      pid, name);
-
    /*
     * Delete certificates for CGI processes...
     */
@@ -1648,6 +1662,9 @@ process_children(void)
 	  else
 	    job->backend = -pid;
 
+          if (job->state_value == IPP_JOB_CANCELED)
+	    status = 0;			/* Ignore errors when canceling jobs */
+
           if (status && job->status >= 0)
 	  {
 	   /*
@@ -1695,7 +1712,39 @@ process_children(void)
 	  break;
 	}
       }
+
+   /*
+    * Show the exit status as needed...
+    */
+
+    if (status == SIGTERM)
+      status = 0;
+
+    if (status)
+    {
+      if (WIFEXITED(status))
+	cupsdLogMessage(CUPSD_LOG_ERROR, "PID %d (%s) stopped with status %d!",
+	                pid, name, WEXITSTATUS(status));
+      else
+	cupsdLogMessage(CUPSD_LOG_ERROR, "PID %d (%s) crashed on signal %d!",
+	                pid, name, WTERMSIG(status));
+
+      if (LogLevel < CUPSD_LOG_DEBUG)
+        cupsdLogMessage(CUPSD_LOG_INFO,
+	                "Hint: Try setting the LogLevel to \"debug\" to find "
+			"out more.");
+    }
+    else
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "PID %d (%s) exited with no errors.",
+                      pid, name);
   }
+
+ /*
+  * If wait*() is interrupted by a signal, tell main() to call us again...
+  */
+
+  if (pid < 0 && errno == EINTR)
+    dead_children = 1;
 }
 
 
@@ -1969,5 +2018,5 @@ usage(int status)			/* O - Exit status */
 
 
 /*
- * End of "$Id: main.c 7299 2008-02-12 01:00:19Z mike $".
+ * End of "$Id: main.c 8301 2009-01-28 22:12:18Z mike $".
  */
