@@ -1,5 +1,5 @@
 /*
- * "$Id: type.c 7201 2008-01-09 04:03:00Z mike $"
+ * "$Id: type.c 8241 2009-01-13 21:02:16Z mike $"
  *
  *   MIME typing routines for the Common UNIX Printing System (CUPS).
  *
@@ -187,8 +187,7 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
     else if (*rule == '+' && current != NULL)
     {
       if (logic != MIME_MAGIC_AND &&
-          current != NULL && current->prev != NULL &&
-	  current->prev->prev != NULL)
+          current != NULL && current->prev != NULL)
       {
        /*
         * OK, we have more than 1 rule in the current tree level...  Make a
@@ -281,8 +280,7 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
       while (isalnum(*rule & 255) && (ptr - name) < (sizeof(name) - 1))
         *ptr++ = *rule++;
 
-      *ptr       = '\0';
-      num_values = 0;
+      *ptr = '\0';
 
       if (*rule == '(')
       {
@@ -407,7 +405,6 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 
 	snprintf(value[0], sizeof(value[0]), "*.%s", name);
 	length[0]  = strlen(value[0]);
-	num_values = 1;
 	op         = MIME_MAGIC_MATCH;
       }
 
@@ -1152,5 +1149,5 @@ patmatch(const char *s,		/* I - String to match against */
 
 
 /*
- * End of "$Id: type.c 7201 2008-01-09 04:03:00Z mike $".
+ * End of "$Id: type.c 8241 2009-01-13 21:02:16Z mike $".
  */
